/*:
 * @target MZ
 * @plugindesc 虚無魔人（2枚目）
 * @author やっつけ
 * @help hidariue2.js
 * 戦闘グラフィックとマップスクロールを、四隅を基準にUIエリアに収める
 * UIエリアはデフォルトの816*624、画面の大きさはそれ以上を想定
 * あんまり極端な値にしなければある程度は大丈夫だと思う
 * UIエリア外の虚無空間はピクチャとかで隠して
 * 公式プラグインのExtraImage.jsや場所移動時にコモンイベント起動するプラグインとかと併用するといいかも
 * 
 * ウィンドウは個別設定めんどいから全部ずらした
 * 
 * ※注意点
 * ・戦闘背景は虚無空間にはみ出ないようにずらしているので、背景中央とUIエリア中央が一致しておらず違和感があるかもしれない
 * 　「右下」、「左下」だとかなり顕著
 * 
 * 公式プラグインExtraImage.js使えばいらないけど、戦闘開始時虚無空間に画像を敷く機能も用意
 * アドレス例　：img/tilesets/
 * ファイル名例：World_C
 * 
 * 絶対どっかでボロが出るか他プラグインと衝突するけどおこらないでね
 * 
 * 利用について
 * 用途・改変・再配布・権利表示に制限なし
 * 
 * 生まれたばかりのプラグインです。
 * 応援して下さいね☆
 * @url https://www.yahoo.co.jp/
 * 
 * @param 寄せ方向
 * @desc 四隅のどこに寄せるか
 * @default 0
 * @type select
 * @option ┏左上┐
 * @value 0
 * @option ┌右上┓
 * @value 1
 * @option ┗左下┘　
 * @value 2
 * @option └右下┛
 * @value 3
 * @param ウィンドウずらし
 * @desc ウィンドウが全部寄せ方向に沿ってずれる　全部って言ったら全部
 * @default true
 * @type boolean
 * @param 戦闘画面外画像アドレス
 * @desc 戦闘中の虚無空間に表示する画像のアドレス
 * @default 
 * @type text
 * @param 戦闘画面外画像ファイル名
 * @desc 戦闘中の虚無空間に表示する画像のファイル名　表示しない場合は空欄
 * @default 
 * @type text
*/
(() => {
    const PN = 'hidariue2';
    const param = PluginManager.parameters(PN);
    const uindou = param['ウィンドウずらし'];
    const haikeiA = param['戦闘画面外画像アドレス'];
    const haikeiB = param['戦闘画面外画像ファイル名'];
    var kijun = param['寄せ方向'];
    /* ウィンドウを全部ずらす */
    Scene_Base.prototype.createWindowLayer = function () {
        this._windowLayer = new WindowLayer();
        if (uindou == 'true') {
            if (kijun == 0) {
                this._windowLayer.x = 4;
                this._windowLayer.y = 4;
            } else if (kijun == 1) {
                this._windowLayer.x  = Graphics.width - Graphics.boxWidth-4;
                this._windowLayer.y = 4;
            } else if (kijun == 2) {
                this._windowLayer.x = 4;
                this._windowLayer.y = Graphics.height - Graphics.boxHeight - 4;
            } else {
                this._windowLayer.x = Graphics.width - Graphics.boxWidth - 4;
                this._windowLayer.y = Graphics.height - Graphics.boxHeight - 4;
            };
        } else {
            this._windowLayer.x = (Graphics.width - Graphics.boxWidth) / 2;
            this._windowLayer.y = (Graphics.height - Graphics.boxHeight) / 2;
        };
        this.addChild(this._windowLayer);
    };

    /* バトルキャラクターのグラフィックをずらす */
    Spriteset_Battle.prototype.createBattleField = function () {
        const width = (Graphics.boxWidth + 8);
        const height = (Graphics.boxHeight + 8);
        var xx;
        var yy;
        if (kijun == 0) {
            xx = 4;
            yy = 4;
        } else if (kijun == 1) {
            xx  = Graphics.width - Graphics.boxWidth-4;
            yy = 4;
        } else if (kijun == 2) {
            xx = 4;
            yy = Graphics.height - Graphics.boxHeight - 4;
        } else {
            xx = Graphics.width - Graphics.boxWidth - 4;
            yy = Graphics.height - Graphics.boxHeight - 4;
        };
        const x=xx;
        const y=yy;
        this._battleField = new Sprite();
        this._battleField.setFrame(0, 0, width, height);
        this._battleField.x = x;
        this._battleField.y = y - this.battleFieldOffsetY();
        this._baseSprite.addChild(this._battleField);
        this._effectsContainer = this._battleField;
    };

    /* 虚無空間にはみ出さないように戦闘背景をずらす */
    Sprite_Battleback.prototype.adjustPosition = function () {
        this.width = Math.floor((1000 * (Graphics.boxWidth + 8)) / 816);
        this.height = Math.floor((740 * (Graphics.boxHeight + 8)) / 624);
        if (kijun == 0 || kijun == 2) {
            this.x = ((Graphics.boxWidth + 8) - this.width);
        } else {
            this.x = (Graphics.width - Graphics.boxWidth - 8);
        };
        if ($gameSystem.isSideView()) {
            if (kijun == 0 || kijun == 1) {
                this.y = (Graphics.boxHeight + 8) - this.height;
            } else {
                this.y = (Graphics.height - Graphics.boxHeight - 8);
            };
        } else {
            this.y = 0;
        };
        const ratioX = this.width / this.bitmap.width;
        const ratioY = this.height / this.bitmap.height;
        const scale = Math.max(ratioX, ratioY, 1.0);
        this.scale.x = scale;
        this.scale.y = scale;
    };

    /* 戦闘画面外の虚無空間に画像を貼る  */
    Spriteset_Battle.prototype.createBackground = function () {
        this._backgroundFilter = new PIXI.filters.BlurFilter();
        this._backgroundSprite = new Sprite();

        if (haikeiB) {
            this._backgroundSprite.bitmap = ImageManager.loadBitmap(haikeiA, haikeiB);
        } else {
            this._backgroundSprite.bitmap = SceneManager.backgroundBitmap();
            this._backgroundSprite.filters = [this._backgroundFilter];
        };
        this._baseSprite.addChild(this._backgroundSprite);
    };

    /* マップをずらす */
    /* マスを数える関数の定義 */
    Game_Map.prototype.screenTileXX = function () {
        return Math.round(((Graphics.boxWidth + 8) / this.tileWidth()) * 16) / 16;
    };
    Game_Map.prototype.screenTileYY = function () {
        return Math.round(((Graphics.boxHeight + 8) / this.tileHeight()) * 16) / 16;
    };
    /* カメラ位置ずらし */
    Game_Player.prototype.centerX = function () {
        if (kijun == 0 || kijun == 2) {
            return ($gameMap.screenTileXX() - 1) / 2;
        } else {
            return (($gameMap.screenTileX() + $gameMap.screenTileXX() * 0.5) - 1) / 2;
        };
    };
    Game_Player.prototype.centerY = function () {
        if (kijun == 0 || kijun == 1) {
            return ($gameMap.screenTileYY() - 1) / 2;
        } else {
            return ($gameMap.screenTileY() + $gameMap.screenTileYY() * 0.5 - 1) / 2;
        };

    };
    /* マップ呼び出し時のずらし */
    Game_Map.prototype.setDisplayPos = function (x, y) {
        if (this.isLoopHorizontal()) {
            this._displayX = x.mod(this.width());
            this._parallaxX = x;
        } else {
            const endX = this.width() - this.screenTileXX();
            if (kijun == 0 || kijun == 2) {
                this._displayX = (endX < 0 ? endX / 2 : x.clamp(0, endX));
            } else {
                this._displayX = (endX < 0 ? endX / 2 : x.clamp(0, endX) + this.screenTileXX() - this.screenTileX());
            };

            this._parallaxX = this._displayX;
        }
        if (this.isLoopVertical()) {
            this._displayY = y.mod(this.height());
            this._parallaxY = y;
        } else {
            const endY = this.height() - this.screenTileYY();
            if (kijun == 0 || kijun == 1) {
                this._displayY = endY < 0 ? endY / 2 : y.clamp(0, endY);
            } else {
                this._displayY = endY < 0 ? endY / 2 : y.clamp(0, endY) + this.screenTileYY() - this.screenTileY();
            };

            this._parallaxY = this._displayY;
        }
    };
    /* スクロールの調整 */
    Game_Map.prototype.scrollDown = function (distance) {
        if (this.isLoopVertical()) {
            this._displayY += distance;
            this._displayY %= $dataMap.height;
            if (this._parallaxLoopY) {
                this._parallaxY += distance;
            }
        } else if (this.height() >= this.screenTileYY()) {
            const lastY = this._displayY;
            if (kijun == 0 || kijun == 1) {
                this._displayY = Math.min(
                    this._displayY + distance,
                    this.height() - this.screenTileYY()
                );
            } else {
                this._displayY = Math.min(
                    this._displayY + distance,
                    this.height() - this.screenTileY()
                );
            };

            this._parallaxY += this._displayY - lastY;
        }
    };
    Game_Map.prototype.scrollLeft = function (distance) {
        if (this.isLoopHorizontal()) {
            this._displayX += $dataMap.width - distance;
            this._displayX %= $dataMap.width;
            if (this._parallaxLoopX) {
                this._parallaxX -= distance;
            }
        } else if (this.width() >= this.screenTileXX()) {
            const lastX = this._displayX;
            if (kijun == 0 || kijun == 2) {
                this._displayX = Math.max(this._displayX - distance, 0);
            } else {
                this._displayX = Math.max(this._displayX - distance, this.screenTileXX() - this.screenTileX());
            };

            this._parallaxX += this._displayX - lastX;
        }
    };
    Game_Map.prototype.scrollRight = function (distance) {
        if (this.isLoopHorizontal()) {
            this._displayX += distance;
            this._displayX %= $dataMap.width;
            if (this._parallaxLoopX) {
                this._parallaxX += distance;
            }
        } else if (this.width() >= this.screenTileXX()) {
            const lastX = this._displayX;
            if (kijun == 0 || kijun == 2) {
                this._displayX = Math.min(
                    this._displayX + distance,
                    this.width() - this.screenTileXX(),
                );
            } else {
                this._displayX = Math.min(
                    this._displayX + distance,
                    this.width() - this.screenTileX(),
                );
            };
            this._parallaxX += this._displayX - lastX;
        }
    };
    Game_Map.prototype.scrollUp = function (distance) {
        if (this.isLoopVertical()) {
            this._displayY += $dataMap.height - distance;
            this._displayY %= $dataMap.height;
            if (this._parallaxLoopY) {
                this._parallaxY -= distance;
            }
        } else if (this.height() >= this.screenTileYY()) {
            const lastY = this._displayY;
            if (kijun == 0 || kijun == 1) {
                this._displayY = Math.max(this._displayY - distance, 0);
            } else {
                this._displayY = Math.max(this._displayY - distance, this.screenTileYY() - this.screenTileY());
            };


            this._parallaxY += this._displayY - lastY;
        }
    };

})();